package opengl;

import static opengl.GL.*;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import main.SimpleTriangle;

/**
 *
 * @author Sascha Kolodzey, Nico Marniok
 */
public class ShaderProgram {   
    public static final int ATTR_POS = 0;
    public static final int ATTR_TEX = 1;
    public static final int ATTR_NORMAL = 2;
    public static final int ATTR_TANGENT = 3;
    public static final int ATTR_COLOR = 3;
    
    private int id;
    
    public ShaderProgram(String vertexShader, String fragmentShader) {
        id = glCreateProgram();
        
        String info = this.compileShader(GL_VERTEX_SHADER, vertexShader);
        if(info != null) {
            System.out.println(vertexShader + " Log:\n" + info);
        }
                
        info = this.compileShader(GL_FRAGMENT_SHADER, fragmentShader);
        if(info != null) {
            System.out.println(fragmentShader + " Log:\n" + info);
        }
        
        glBindAttribLocation(id, ATTR_POS, "vs_in_pos");
        glBindAttribLocation(id, ATTR_TEX, "vs_in_tex");
        glBindAttribLocation(id, ATTR_NORMAL, "vs_in_normal");
        glBindAttribLocation(id, ATTR_TANGENT, "vs_in_tangent");
        glBindAttribLocation(id, ATTR_COLOR, "vs_in_color");
        
        glLinkProgram(id);
        
        info = glGetProgramInfoLog(id, 1024);
        if(info != null && !info.isEmpty()) {
            System.out.println("Program Log:\n" + info);
        }
    }
    
    private String compileShader(int type, String filename) {
        if(filename != null) {
            int shaderID = glCreateShader(type);
            glAttachShader(id, shaderID);
            glShaderSource(shaderID, this.getShaderCode(filename));
            glCompileShader(shaderID);
            String info = glGetShaderInfoLog(shaderID, 1024);
            return info != null && !info.isEmpty() ? info : null;
        } else {
            return null;
        }
    }
    
    private String getShaderCode(String filename) {
        BufferedReader reader = null;
        String source = null;
        try {
            reader = new BufferedReader(new FileReader(filename));
            source = "";
            String line = "";
            while((line = reader.readLine()) != null) {
                source += line + "\n";
            }
        } catch (IOException ex) {
            Logger.getLogger(SimpleTriangle.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                reader.close();
            } catch (IOException ex) {
                Logger.getLogger(SimpleTriangle.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return source;
    }

    public int getID() {
        return id;
    }
}
